/*---------------------------------------------------------------------------*/

/*!  \file  blend.c
     \brief  definitions of the polymer blends.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2007 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#include "common.h"

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#ifdef HAVE_MATH_H
#include <math.h>
#endif

#include "xmalloc.h"
#include "blend.h"

/*---------------------------------------------------------------------------*/
polymer_blend *allocate_polymer_blend(int n)
{
    int i;
    polymer_blend *blend;

    blend = xmalloc(sizeof(polymer_blend));

    blend->n = n;
    blend->m = xmalloc(sizeof(int) * n);
    blend->next = xmalloc(sizeof(int) * n);
    blend->epsilon = xmalloc(sizeof(double *) * n);
    blend->epsilon[0] = xmalloc(sizeof(double) * n * n);

    for(i = 1;i < n;i++)
    {
        blend->epsilon[i] = blend->epsilon[0] + i * n;
    }

    return blend;
}

/*---------------------------------------------------------------------------*/
void free_polymer_blend(polymer_blend *blend)
{
    if(blend == NULL)
        return;

    free(blend->epsilon[0]);
    free(blend->epsilon);
    free(blend->next);
    free(blend->m);

    free(blend);
}

/*---------------------------------------------------------------------------*/
void calculate_lambda0(polymer_blend *blend,box_geometry *geometry)
{
    int i,m_tot,sigma0;

    if(geometry->dimension != 2)
    {
        blend->lambda0 = 1.0;
        return;
    }

    m_tot = 0;
    for(i = 0;i < blend->n;i++)
    {
        m_tot += blend->m[i];
    }

    sigma0 = m_tot / (geometry->l[0] * geometry->l[1]);

    blend->lambda0 = sqrt(sigma0);
}

/*---------------------------------------------------------------------------*/
