/*---------------------------------------------------------------------------*/

/*!  \file  dxwrite.c
     \brief  output the position data as OpenDX .  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2007 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/

/*  DO NOT EDIT THIS FILE. This file is generated from dxwrite.c.m4.  */

#include "common.h"

#include <stdio.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_STRING_H
#include <string.h>
#endif

#include "xmalloc.h"
#include "dxwrite.h"

/*---------------------------------------------------------------------------*/
#if (SIZEOF_FLOAT == 4)
static void write_positions2_dx_binary(char *filename,
                                         soft_particles2 *particles,
                                         box_geometry *geometry);
static void write_box2_dx(FILE *fp,box_geometry *geometry);

static void write_positions3_dx_binary(char *filename,
                                         soft_particles3 *particles,
                                         box_geometry *geometry);
static void write_box3_dx(FILE *fp,box_geometry *geometry);
#else

#endif

/*---------------------------------------------------------------------------*/
void write_positions_dx(char *filename,soft_particles *particles,
                        box_geometry *geometry)
{
    if((filename == NULL) || (strlen(filename) == 0))
        return;

#if (SIZEOF_FLOAT == 4)
        /*  IEEE floating number  (binary)  */
        switch(particles->dimension)
    {
        case 2:
            write_positions2_dx_binary(filename,&(particles->particles2),geometry);
            break;

        case 3:
            write_positions3_dx_binary(filename,&(particles->particles3),geometry);
            break;

        default:
            break;
    }
#else
            /*  ASCII text (if IEEE floating number is not available)  */
    switch(particles->dimension)
    {
        case 2:
/*                 write_position_dx_text_2d(filename,particles); */
            break;

        case 3:
/*                 write_position_dx_text_3d(filename,particles); */
            break;

        default:
            return;
    }
#endif
}

/*---------------------------------------------------------------------------*/
#if (SIZEOF_FLOAT == 4)

/*---------------------------------------------------------------------------*/
void write_positions2_dx_binary(char *filename,soft_particles2 *particles,
                                                   box_geometry *geometry)
{
    FILE *fp;
    int i,j;
    int offset;
    int m_max;
    float *work;
    char *endian;

#ifdef WORDS_BIGENDIAN
    endian = "msb";
#else
    endian = "lsb";
#endif

    m_max = 0;
    for(i = 0;i < particles->n;i++)
    {
        if(particles->m[i] > m_max)
            m_max = particles->m[i];
    }
    work = xmalloc(sizeof(float) * 2 * m_max);

    fp = fopen(filename,"w");


    write_box2_dx(fp,geometry);


        /*  write the data of positions of soft particles.  */
    offset = 0;
    for(i = 0;i < particles->n;i++)
    {
        fprintf(fp,"object \"data_position%d\" class array type float rank 1 shape 2 items %d %s ieee data %d\n",
                i,particles->m[i],endian,offset);
        fprintf(fp,"\n");

        offset += 2 * particles->m[i] * sizeof(float);
    }


    for(i = 0;i < particles->n;i++)
    {
        fprintf(fp,"object \"position%d\" class field\n",i);
        fprintf(fp,"component \"positions\" value \"data_position%d\"\n",i);
        fprintf(fp,"\n");
    }

    fprintf(fp,"end\n");

    for(i = 0;i < particles->n;i++)
    {
        for(j = 0;j < particles->m[i];j++)
        {
            work[2 * j + 0] = particles->r[i][j][0];
work[2 * j + 1] = particles->r[i][j][1];
        }
        fwrite(work,sizeof(float),2 * particles->m[i],fp);
    }


    fclose(fp);

    free(work);
}

/*---------------------------------------------------------------------------*/
void write_box2_dx(FILE *fp,box_geometry *geometry)
{
                /*  write the box data  */
    fprintf(fp,"object \"box_data\" class gridpositions counts 2 2\n");
           fprintf(fp,"origin 0.0 0.0\n");
           fprintf(fp,"delta %f 0.0\n",geometry->l[0]);
           fprintf(fp,"delta 0.0 %f\n",geometry->l[1]);
    fprintf(fp,"\n");
    fprintf(fp,"object \"box\" class field\n");
    fprintf(fp,"component \"positions\" value \"box_data\"\n");
    fprintf(fp,"\n");

    fprintf(fp,"object \"delta_l\" class array type float rank 0 items 1 data follows\n");
    fprintf(fp,"%f\n",geometry->delta_l);
    fprintf(fp,"\n");
}

/*---------------------------------------------------------------------------*/
void write_positions3_dx_binary(char *filename,soft_particles3 *particles,
                                                   box_geometry *geometry)
{
    FILE *fp;
    int i,j;
    int offset;
    int m_max;
    float *work;
    char *endian;

#ifdef WORDS_BIGENDIAN
    endian = "msb";
#else
    endian = "lsb";
#endif

    m_max = 0;
    for(i = 0;i < particles->n;i++)
    {
        if(particles->m[i] > m_max)
            m_max = particles->m[i];
    }
    work = xmalloc(sizeof(float) * 3 * m_max);

    fp = fopen(filename,"w");


    write_box3_dx(fp,geometry);


        /*  write the data of positions of soft particles.  */
    offset = 0;
    for(i = 0;i < particles->n;i++)
    {
        fprintf(fp,"object \"data_position%d\" class array type float rank 1 shape 3 items %d %s ieee data %d\n",
                i,particles->m[i],endian,offset);
        fprintf(fp,"\n");

        offset += 3 * particles->m[i] * sizeof(float);
    }


    for(i = 0;i < particles->n;i++)
    {
        fprintf(fp,"object \"position%d\" class field\n",i);
        fprintf(fp,"component \"positions\" value \"data_position%d\"\n",i);
        fprintf(fp,"\n");
    }

    fprintf(fp,"end\n");

    for(i = 0;i < particles->n;i++)
    {
        for(j = 0;j < particles->m[i];j++)
        {
            work[3 * j + 0] = particles->r[i][j][0];
work[3 * j + 1] = particles->r[i][j][1];
work[3 * j + 2] = particles->r[i][j][2];
        }
        fwrite(work,sizeof(float),3 * particles->m[i],fp);
    }


    fclose(fp);

    free(work);
}

/*---------------------------------------------------------------------------*/
void write_box3_dx(FILE *fp,box_geometry *geometry)
{
                /*  write the box data  */
    fprintf(fp,"object \"box_data\" class gridpositions counts 2 2 2\n");
           fprintf(fp,"origin 0.0 0.0 0.0\n");
           fprintf(fp,"delta %f 0.0 0.0\n",geometry->l[0]);
           fprintf(fp,"delta 0.0 %f 0.0\n",geometry->l[1]);
           fprintf(fp,"delta 0.0 0.0 %f\n",geometry->l[2]);
    fprintf(fp,"\n");
    fprintf(fp,"object \"box\" class field\n");
    fprintf(fp,"component \"positions\" value \"box_data\"\n");
    fprintf(fp,"\n");

    fprintf(fp,"object \"delta_l\" class array type float rank 0 items 1 data follows\n");
    fprintf(fp,"%f\n",geometry->delta_l);
    fprintf(fp,"\n");
}

/*---------------------------------------------------------------------------*/
#else

/*---------------------------------------------------------------------------*/
#endif


/*---------------------------------------------------------------------------*/
