dnl ---------------------------------------------------------------------------
dnl
dnl  particles.c.m4
dnl
dnl ---------------------------------------------------------------------------
dnl
dnl  Copyright (C) 2007 Takashi Uneyama
dnl
dnl  This program is free software; you can redistribute it and/or modify
dnl  it under the terms of the GNU General Public License as published by
dnl  the Free Software Foundation; either version 2, or (at your option)
dnl  any later version.
dnl
dnl  This program is distributed in the hope that it will be useful,
dnl  but WITHOUT ANY WARRANTY; without even the implied warranty of
dnl  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
dnl  GNU General Public License for more details.
dnl
dnl  You should have received a copy of the GNU General Public License
dnl  along with this program; if not, write to the Free Software
dnl  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
dnl  USA.
dnl
dnl ---------------------------------------------------------------------------
include(`dimension.m4')`'dnl
GENERATE_HEADER(dxwrite.c,
                output the position data as OpenDX format.)

#include "common.h"

#include <stdio.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_STRING_H
#include <string.h>
#endif

#include "xmalloc.h"
#include "dxwrite.h"

/*---------------------------------------------------------------------------*/
#if (SIZEOF_FLOAT == 4)
PROTOTYPE_FUNCTIONS(`dnl
static void write_positions@D@_dx_binary(char *filename,
                                         soft_particles@D@ *particles,
                                         box_geometry *geometry);
static void write_box@D@_dx(FILE *fp,box_geometry *geometry);'`')
#else

#endif

/*---------------------------------------------------------------------------*/
void write_positions_dx(char *filename,soft_particles *particles,
                        box_geometry *geometry)
{
    if((filename == NULL) || (strlen(filename) == 0))
        return;

#if (SIZEOF_FLOAT == 4)
        /*  IEEE floating number format (binary)  */
    SWITCH_FUNCTION(particles->dimension,
                    write_positions@D@_dx_binary(filename,&(particles->particles@D@),geometry);)
#else
            /*  ASCII text (if IEEE floating number is not available)  */
    switch(particles->dimension)
    {
        case 2:
/*                 write_position_dx_text_2d(filename,particles); */
            break;

        case 3:
/*                 write_position_dx_text_3d(filename,particles); */
            break;

        default:
            return;
    }
#endif
}

/*---------------------------------------------------------------------------*/
#if (SIZEOF_FLOAT == 4)

/*---------------------------------------------------------------------------*/
EXPAND_FUNCTIONS(`dnl
void write_positions@D@_dx_binary(char *filename,soft_particles@D@ *particles,
                                                   box_geometry *geometry)
{
    FILE *fp;
    int i,j;
    int offset;
    int m_max;
    float *work;
    char *endian;

#ifdef WORDS_BIGENDIAN
    endian = "msb";
#else
    endian = "lsb";
#endif

    m_max = 0;
    for(i = 0;i < particles->n;i++)
    {
        if(particles->m[i] > m_max)
            m_max = particles->m[i];
    }
    work = xmalloc(sizeof(float) * @D@ * m_max);

    fp = fopen(filename,"w");


    write_box@D@_dx(fp,geometry);


        /*  write the data of positions of soft particles.  */
    offset = 0;
    for(i = 0;i < particles->n;i++)
    {
        fprintf(fp,"object \"data_position%d\" class array type float rank 1 shape @D@ items %d %s ieee data %d\n",
                i,particles->m[i],endian,offset);
        fprintf(fp,"\n");

        offset += @D@ * particles->m[i] * sizeof(float);
    }


    for(i = 0;i < particles->n;i++)
    {
        fprintf(fp,"object \"position%d\" class field\n",i);
        fprintf(fp,"component \"positions\" value \"data_position%d\"\n",i);
        fprintf(fp,"\n");
    }

    fprintf(fp,"end\n");

    for(i = 0;i < particles->n;i++)
    {
        for(j = 0;j < particles->m[i];j++)
        {
            VECTOR@D@(work[@D@ * j + @@] = particles->r[i][j][@@];)
        }
        fwrite(work,sizeof(float),@D@ * particles->m[i],fp);
    }


    fclose(fp);

    free(work);
}

/*---------------------------------------------------------------------------*/
void write_box@D@_dx(FILE *fp,box_geometry *geometry)
{
                /*  write the box data  */
    ifelse(@D@,2,
           fprintf(fp,"object \"box_data\" class gridpositions counts 2 2\n");
           fprintf(fp,"origin 0.0 0.0\n");
           fprintf(fp,"delta %f 0.0\n",geometry->l[0]);
           fprintf(fp,"delta 0.0 %f\n",geometry->l[1]);,
           @D@,3,
           fprintf(fp,"object \"box_data\" class gridpositions counts 2 2 2\n");
           fprintf(fp,"origin 0.0 0.0 0.0\n");
           fprintf(fp,"delta %f 0.0 0.0\n",geometry->l[0]);
           fprintf(fp,"delta 0.0 %f 0.0\n",geometry->l[1]);
           fprintf(fp,"delta 0.0 0.0 %f\n",geometry->l[2]);)
    fprintf(fp,"\n");
    fprintf(fp,"object \"box\" class field\n");
    fprintf(fp,"component \"positions\" value \"box_data\"\n");
    fprintf(fp,"\n");

    fprintf(fp,"object \"delta_l\" class array type float rank 0 items 1 data follows\n");
    fprintf(fp,"%f\n",geometry->delta_l);
    fprintf(fp,"\n");
}'`')

/*---------------------------------------------------------------------------*/
#else

/*---------------------------------------------------------------------------*/
#endif


/*---------------------------------------------------------------------------*/
