/*---------------------------------------------------------------------------*/

/*!  \file  force.h
     \brief  calculate the random and interaction forces.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2007,2008 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#ifndef INCLUDE_FORCE_H
#define INCLUDE_FORCE_H

#include "vector.h"
#include "particles.h"
#include "celllist.h"

/*---------------------------------------------------------------------------*/
/*!  \brief  generate random forces acting on soft particles.
     \param  particles  soft particles  */
void generate_random_force(soft_particles *particles);
/*!  \brief  cleaer interaction forces acting on soft particles.
     \param  particles  soft particles  */
void clear_interaction_force(soft_particles *particles);
/*!  \brief  accumulate the tethering interaction forces acting on soft particles.
     \param  particles  soft particles
     \param  blend  polymer blend
     \param  geometry  box geometry  */
void accumulate_tethering_interaction_force(soft_particles *particles,
                                            polymer_blend *blend,
                                            box_geometry *geometry);
void accumulate_tethering_interaction_force_star(soft_particles *particles,
                                                 polymer_blend *blend,
                                                 box_geometry *geometry);
/*!  \brief  accumulate the site interaction forces acting on soft particles.
     \param  particles  soft particles
     \param  celllist  linked cell list
     \param  blend  polymer blend
     \param  geometry  box geometry  */
void accumulate_site_interaction_force(soft_particles *particles,
                                       linked_cell_list *celllist,
                                       polymer_blend *blend,
                                       box_geometry *geometry);
void accumulate_site_interaction_force_star(soft_particles *particles,
                                            linked_cell_list *celllist,
                                            polymer_blend *blend,
                                            box_geometry *geometry);
/*!  \brief  calculate the tethering and site interaction forces (without noise) acting on soft particles.
     \param  particles  soft particles
     \param  celllist  linked cell list
     \param  blend  polymer blend
     \param  geometry  box geometry  */
void calculate_interaction_force_without_noise(soft_particles *particles,
                                               linked_cell_list *celllist,
                                               polymer_blend *blend,
                                               box_geometry *geometry);
/*!  \brief  calculate the stress tensor.
     \param  particles  soft particles
     \param  celllist  linked cell list
     \param  blend  polymer blend
     \param  geometry  box geometry  */
void calculate_stress_tensor(soft_particles *particles,
                             linked_cell_list *celllist,
                             polymer_blend *blend,
                             box_geometry *geometry);

/*---------------------------------------------------------------------------*/
#endif

/*---------------------------------------------------------------------------*/
