/*---------------------------------------------------------------------------*/

/*!  \file  textread.h
     \brief  read the position and force data from gzipped ascii files.  */

/*---------------------------------------------------------------------------*/

/* Copyright (C) 2008,2020 Takashi Uneyama  */

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/*---------------------------------------------------------------------------*/
#ifndef INCLUDE_TEXTREAD_H
#define INCLUDE_TEXTREAD_H

/*---------------------------------------------------------------------------*/
/*!  \brief  read the stress tensor from files sequentially.
     \param  filename  stress data file name
     \param  sigma  stress tensor
     \param  dimension  dimension of the system
     \param  m  number of files  */
void read_stress_text_sequential(char *filename,double ***sigma,int dimension,
                                 int m);
/*!  \brief  read the box geometry from \a filename.
     \param  filename  file name
     \param  dimension  dimension of the system
     \param  l  lengthes of box edges
     \param  delta_l  gap between cells  */
void read_geometry_text(char *filename,int *dimension,double *l,
                        double *delta_l);
/*!  \brief  read the particle positions from \a filename.
     \param  filename  file name
     \param  r  positions of soft particles
     \param  dimension  dimension of the system
     \param  n  number of components
     \param  m  number of soft particles  */
void read_positions_text(char *filename,double ***r,int *dimension,
                         int *n,int **m);

/*---------------------------------------------------------------------------*/
#endif

/*---------------------------------------------------------------------------*/
